@@ -16,7 +16,7 @@ _additional_titles = ["med.", "Dipl.", "Psych.", "Apl.", "Ass.", "rer.", "nat.",
                       "M.B.CH.B", "Sanitätsrat", "dr/Universität", "-Ing.", "DrPH", "jur.", "Cirug./EC",
                       "professor/NGMU", "Ing.", "M.D./Ain", "Universität", "M.D./SYR", "EC", "pocetny", "vet.",
                       "DrM/Univ.", "Cirug.", "drs.", "h.c.", "d-r", "M. A.", "M. D./Univ.", "Diplom-Sozialwirt",
-                      "B.Ch.", "M.B.", "scient.", "pth.", "Dipl.-Psych.",
+                      "B.Ch.", "M.B.", "scient.", "pth.", "Dipl.-Psych.","Bac.",
                       "D.O.", "DAAO", "Ch.B./Univ.", "HMA", "Diplom-Psychologe", "physiol.", "P.H.", "hab.", "M.B.B.S.",
                       "hum.", "M.D.", "/OAK", "vrac", "Diplom-Biologe", "dr/Univ.", "Docteur", "Médecine/Univ.",
                       "B.M.D.", "doktor", "medicine", "/Mediz.", "M.B.", "M.D./Univ.", "M.B.,B.Ch.(ET)", "M.S.P.",
@@ -28,7 +28,7 @@ _additional_titles = ["med.", "Dipl.", "Psych.", "Apl.", "Ass.", "rer.", "nat.",
                       "Oradea", "Doc.", "Deptl.", "MB.Ch.B.", "OMR", "OMD", "Fachärztin", "D.A.L.M.", "DEAA", "jun.",
                       "Assistenzärztin", "Klinikdirektor", "Soz.", "M.B.A.", "FEAN", "FNCS", "IFAANS", "M.S.",
                       "Med.-Dir.", "Sportwiss.", "OTA", "a.D.", "medic/R", "B.S.", "M.B.", "Ch.B.", "D.A.L.M.", "MaHM",
-                      "B.A.", "UIPA", "universae/Semmelweis", "Doktor-e-reste-ye", "dr.sc.", "VUB", "Doktoru/Univ.",
+                      "B.A.", "UIPA", "universae/Semmelweis", "Doktor-e-reste-ye", "dr.sc.", "VUB", "DoktoFru/Univ.",
                       "Doktor-i", "pezeski/Univ.", "medic/Univ.", "medicinae", "universae", "San.-Rat", "Medizinalrat",
                       "Dott/Univ.", "M.D./Univ.Damaskus", "Ped.", "/I.N.E.S.S.", "LL.M.", "Ltd.", "Doktor-e", "M.sc.",
                       "Lic./Univ.", "pezeski/Univ.", "Odessa/UdSSR", "Medizinaldirektor", "Doktore", "pezeski/",
@@ -41,7 +41,7 @@ _additional_titles = ["med.", "Dipl.", "Psych.", "Apl.", "Ass.", "rer.", "nat.",
 _surname_titles = ["Von ", "von ", "van ", "Van ", "dos ", "Dos "]
 
 # some titles are actual names, the result would be None, therefore we remove some
-_remove_title = ["mahdi", "graf", "singer", "lama", "pastor", "imam", "jun", "baba","baron"]
+_remove_title = ["mahdi", "graf", "singer", "lama", "pastor", "imam", "jun", "baba", "baron"]
 
 constants = Constants()
 for a in _additional_titles:
@@ -58,12 +58,14 @@ def normalize_title(inp: str) -> Optional[str]:
     """
     if inp is None:
         return None
-    return inp.replace("Professor", "Prof.") \
-        .replace("doktor medicine", "Dr. med.") \
-        .replace("Doktor", "Dr.") \
-        .replace("dr/Universität", "Dr. Univ.") \
+    return inp.replace("dr/Universität", "Dr. Univ.") \
+        .replace("Universitätsprofessor", "Prof. Univ.") \
         .replace("Universitätsprof.", "Prof. Univ.") \
         .replace("Universität", "Univ.") \
+        .replace("Professor", "Prof.") \
+        .replace("prof.", "Prof.") \
+        .replace("doktor medicine", "Dr. med.") \
+        .replace("Doktor", "Dr.") \
         .replace("Doctor", "Dr.") \
         .replace("dr.", "Dr.") \
         .replace("medic.", "med.") \
@@ -72,13 +74,19 @@ def normalize_title(inp: str) -> Optional[str]:
         .replace("Privatdozent", "PD") \
         .replace("Privat - Dozent", "PD") \
         .replace("Priv.-Doz.", "PD") \
+        .replace("Priv.Doz.", "PD") \
+        .replace("Priv. Doz.", "PD") \
+        .replace("PDDr.", "PD Dr.") \
         .replace(" / ", " ") \
         .replace("./", ". ") \
         .replace(". /", ". ") \
         .replace(" - ", " ") \
         .replace(".-", ".") \
+        .replace(". -", ". ") \
+        .replace('Dr.med.', 'Dr. med.').replace('Dr', 'Dr.').replace('Prof', 'Prof.')\
+        .replace('..', '.').replace('.','. ') \
         .replace("  ", " ") \
-        .replace(". -", ". ").strip()
+        .strip()
 
 
 def person_title(inp: str, normalize=True) -> Optional[str]:
@@ -212,7 +220,7 @@ def parse_name(inp: str) -> Optional[HumanName]:
         return None
 
     if " " not in inp and "." in inp:
-        #if there are missing spaces, split them up
+        # if there are missing spaces, split them up
         inp = inp.replace(".", ". ")
 
     inp = inp.title()
@@ -334,7 +342,25 @@ def match_name(own: str, other: str) -> bool:
     hn_own.last = " ".join(hn_own.last_list)
 
     # if the last names doesnt match, we skip here
-    if [on.lower() for on in hn_own.last_list] != [on.lower() for on in hn_other.last_list]:
+    own_lasts = " ".join([on.lower() for on in hn_own.last_list])
+    other_lasts = " ".join([on.lower() for on in hn_other.last_list])
+
+    # compound surnames
+    if "-" in own_lasts or "-" in other_lasts:
+        own_lasts_splitted = own_lasts.split("-")
+        other_lasts_splitted = other_lasts.split("-")
+        matches = 0
+        for o in own_lasts_splitted:
+            for ot in other_lasts_splitted:
+                if o == ot or distance(o, ot) <= 1 and (len(o) >= 5 or len(ot) >= 5):
+                    matches += 1
+        for o in reversed(own_lasts_splitted):
+            for ot in other_lasts_splitted:
+                if o == ot or distance(o, ot) <= 1 and (len(o) >= 5 or len(ot) >= 5):
+                    matches += 1
+        if matches < 2:
+            return False
+    elif own_lasts[0] != other_lasts[0] or (own_lasts != other_lasts and distance(own_lasts, other_lasts) > 1):
         return False
 
     def _match_name_list(name: str, other: List[str]):
@@ -358,24 +384,30 @@ def match_name(own: str, other: str) -> bool:
     # check if the firstnames matches (if one side has no firstname we assume a match
     first_name_matches = True if (hn_own.first == "" or hn_other.first == "") else _compare_names(hn_own.first_list,
                                                                                                   hn_other.first_list)
+    own_first_middles = hn_own.first + hn_own.middle
+    other_first_middles = hn_other.first + hn_other.middle
 
-    if first_name_matches is False:
+    # check if the firstnames+middlename matches (if one side has no firstname we assume a match
+    first_name_matches_fuzzy = own_first_middles.lower() == other_first_middles.lower() or (
+                own_first_middles.startswith(other_first_middles) or other_first_middles.startswith(own_first_middles))
+
+    if first_name_matches is False or first_name_matches_fuzzy is False:
         # if the initials dont match, dont match
-        if hn_own.first[0] != hn_other.first[0]:
+        if (len(hn_own.first) > 0 and len(hn_own.first) > 0) and hn_own.first[0] != hn_other.first[0]:
             return False
 
         # if the names are longer than 5 and start with the same letter we allow tiny typos
         l_distance = distance(hn_own.first, hn_other.first)
-        if l_distance < 2 and hn_own.first[0] == hn_other.first[0] and len(hn_other.first) > 5 and len(hn_own.first) > 5:
+        if l_distance < 2 and (len(hn_other.first) >= 5 or len(hn_own.first) >= 5):
             first_name_matches = True
 
     # if none has middle name its a match
     if len(hn_own.middle_list) == 0 and len(hn_other.middle_list) == 0:
-        return first_name_matches
+        return first_name_matches or first_name_matches_fuzzy
 
     # if only one side has a middle name its a match
     if len(hn_own.middle_list) == 0 and len(hn_other.middle_list) > 0 or len(hn_own.middle_list) > 0 and len(
             hn_other.middle_list) == 0:
-        return first_name_matches
+        return first_name_matches or first_name_matches_fuzzy
 
     return _compare_names(hn_own.middle_list, hn_other.middle_list)
